#!/bin/bash

cd $(dirname $0)
SCRIPTDIR=$(pwd)
echo $SCRIPTDIR

#check browser.done file
if [ ! -f browser.done -a -f browser.ini ]; then
    echo -e "var browser_settings =" `sed -f ini2json.sed browser.ini` > ext/browser_settings.js
    DEF_USER_AGENT="$( cat browser.ini | grep customUserAgent | sed 's/^;.*//' | cut -d'=' -f2 | tail -n1 )"
    touch browser.done
else
   echo "var browser_settings = {}" > "ext/browser_settings.js"
fi

export DISPLAY=:0
. $HOME/.dbus/session-bus/*
export DBUS_SESSION_BUS_ADDRESS

export GTK_IM_MODULE_FILE=$SCRIPTDIR/gtk.immodules
export GTKIM_POS_IMPL=1

DATA_DIR="$SCRIPTDIR/.chromium/"
CACHE_DIR="$DATA_DIR/Default/Cache"
EXTENSION_DIR="$SCRIPTDIR/ext/"
EXTENSION_CACHE_DIR="$DATA_DIR/Default/Local Extension Settings/pppblgnjgngbhkjehblmbfeklkbfamdm/"
EXTENSION_CACHE_LOCK="$EXTENSION_CACHE_DIR/LOCK"
EXTENSION_CACHE_DB="$EXTENSION_CACHE_DIR/000003.log"
EXTENSION_CMD_FILE="$EXTENSION_DIR/cmd_line.js"
CHROME_WINDOW_FLAG="/run/chrome-window"
CACHE_SIZE_MB=60

TOTAL_RAM_KB="$( cat /proc/meminfo | grep MemTotal | awk '{print $2}' )"
[ "$TOTAL_RAM_KB" -lt "250000" ] && CACHE_SIZE_MB=30

if !( df -h $CACHE_DIR 2>/dev/null | sed -n 2p | grep -q ^tmpfs ); then
        mkdir -p $CACHE_DIR
        mount -t tmpfs -o size=$(( $CACHE_SIZE_MB +3 ))M tmpfs $CACHE_DIR
fi

CPU="$( cat /proc/cpuinfo | grep Hardware )"
if ( echo $CPU | grep -q "Freescale i.MX6" ); then
	GPU_FLAGS="--use-gl=egl --enable-zero-copy --disable-accelerated-2d-canvas --disable-es3-gl-context \
		--enable-gpu-compositing --enable-threaded-compositing --gpu-no-context-lost \
		--enable-gpu-memory-buffer-video-frames --enable-native-gpu-memory-buffers"
else
	GPU_FLAGS="--disable-gpu"
fi

CMD_LINE=""

[ -e $CHROME_WINDOW_FLAG ] && exit 0
touch $CHROME_WINDOW_FLAG

declare -A extDB
if [ -e "$EXTENSION_CACHE_DB" ]; then
	while read line; do
		variable="$( echo $line | cut -d"'" -f2 )"
		value="$( echo $line | cut -d"'" -f4 | sed 's:^"\(.*\)"$:\1:' )"
		if ( echo $value | grep '^".*"$' ); then
			value="$( echo $value | sed 's:^"\(.*\)"$:\1:' )"
		elif [ "$value" == "null" ]; then
			continue
		fi

		extDB["$variable"]="$value"
	done < <( LD_LIBRARY_PATH=$SCRIPTDIR/lib $SCRIPTDIR/bin/leveldbutil dump "$EXTENSION_CACHE_DB" | grep put )
fi

CHROME_ARGS="--no-sandbox --extensions-on-chrome-urls --disable-session-crashed-bubble \
	--disable-infobars --overscroll-history-navigation=0 --no-first-run --kiosk \
        --user-data-dir=$DATA_DIR --disk-cache-size=$(( $CACHE_SIZE_MB *1024 *1024 )) $GPU_FLAGS"

# Uncomment to enable verbose log
# CHROME_ARGS="$CHROME_ARGS --enable-logging=stderr --v=1"

# Uncomment to enable remote debugging on port 9222
# CHROME_ARGS="$CHROME_ARGS --remote-debugging-port=9222"

# Uncomment and set <IP> and <PORT> to enable proxy
# CHROME_ARGS="$CHROME_ARGS --proxy-bypass-list=127.0.0.1 --proxy-server=<IP>:<PORT>"

# Non multitouch devices are not recognized as touchscreens. We need to force chromium to believe they are
titscdev=$( /usr/bin/xinput | grep ti-tsc | sed "s:.*id=\([0-9]*\).*:\1:" )
[ -n "$titscdev" ] && CHROME_ARGS="$CHROME_ARGS --touch-devices=$titscdev --touch-events"

tsc2004dev=$( /usr/bin/xinput | grep TSC2004 | sed "s:.*id=\([0-9]*\).*:\1:" )
[ -n "$tsc2004dev" ] && CHROME_ARGS="$CHROME_ARGS --touch-devices=$tsc2004dev --touch-events"

CHROME_ARGS_BG="$CHROME_ARGS --load-extension=$EXTENSION_DIR --no-startup-window"

[ -n "${extDB[customUserAgent]}" ] && USER_AGENT="--user-agent=${extDB[customUserAgent]}"
[ -n "$DEF_USER_AGENT" ] && USER_AGENT="--user-agent=$DEF_USER_AGENT"

# Chromium crash detection triggers every time the panel is powered off without properly closing the applciation. We have
# to clear out this state otherwise Chromium will refuse to load saved settings, probably considering safer this way.
[ -e $DATA_DIR/Default/Preferences ] && sed -i'' 's/"exit_type":"Crashed"/"exit_type":"Normal"/' $DATA_DIR/Default/Preferences

( pidof chromium-bin &>/dev/null ) && daemonRunning=1

if [ ! -n "$daemonRunning" ]; then
	rm -rf $DATA_DIR/Singleton*
	rm -f "$EXTENSION_CACHE_LOCK"
fi

(
	export GTK_IM_MODULE_FILE=

	# Wait for daemon socket and extension cache dir to be created
	while [ ! -e $DATA_DIR/SingletonSocket -o ! -e "$EXTENSION_CACHE_LOCK" ]; do
		sleep 1
	done;

	sleep 1

	# Open a browser window
	LD_LIBRARY_PATH=$SCRIPTDIR/lib \
		./chromium-bin $CHROME_ARGS $@

	rm -f $CHROME_WINDOW_FLAG
) &

if [ -n "$daemonRunning" ]; then
	echo Daemon already running
	sleep 10
	exit 0
fi

# Pass command line to extension
[ -e settings ] && CMD_LINE="$( cat settings )"
CMD_LINE="$CMD_LINE $@"
echo 'var cmd_line = "'$CMD_LINE'";' > $EXTENSION_CMD_FILE
echo 'var inst_dir = "'$SCRIPTDIR'";' >> $EXTENSION_CMD_FILE

# Run the daemon fist to make sure the extension is loaded
LD_LIBRARY_PATH=$SCRIPTDIR/lib \
	./chromium-bin $CHROME_ARGS_BG "$USER_AGENT"

if [ $? -eq "137" -a ! -e $DATA_DIR/SingletonSocket ]; then
	sleep 2
	LD_LIBRARY_PATH=$SCRIPTDIR/lib \
		./chromium-bin $CHROME_ARGS_BG "$USER_AGENT"
fi

rm -f $CHROME_WINDOW_FLAG
